@@ -10,7 +10,7 @@ use UNIVERSAL;
 use Exporter;
 
 
-our $VERSION = '0.13';
+our $VERSION = '0.14';
 our $DEBUG;
 
 our @ISA = qw(Exporter);
@@ -22,6 +22,7 @@ our @EXPORT_OK = qw(
     max
     min
     sum
+    LENGTH
     cast
     upper
     lower
@@ -195,6 +196,11 @@ sub sum {
 }
 
 
+sub LENGTH {
+    return do_function('LENGTH', @_);
+}
+
+
 sub cast {
     return do_function('CAST', @_);
 }
@@ -664,6 +670,10 @@ in {...definition...}. Each table can only be defined once.
 
 
 
+=head2 LENGTH
+
+
+
 =head2 cast
 
 
